from oarepo_model_builder.builders.json_base import JSONBaseBuilder
from oarepo_model_builder.builders.jsonschema import JSONSchemaBuilder
from oarepo_model_builder.stack import ModelBuilderStack


class JSONSchemaDraftsParentBuilder(JSONSchemaBuilder):
    TYPE = "jsonschema_drafts_parent"
    output_file_type = "jsonschema"
    output_file_name = "drafts-parent-schema-file"
    parent_module_root_name = "jsonschemas"

    def begin(self, schema, settings):
        super().begin(schema, settings)
        target_json = {
            "$schema": "http://json-schema.org/draft-07/schema#",
            "$id": f"{schema.current_model.schema_server}{schema.current_model.drafts_parent_schema_name}",
            "type": "object",
            "properties": {
                "id": {
                    "type": "string"
                }
            }
        }
        self.stack = ModelBuilderStack()
        self.stack.push(None, target_json)



from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder


class InvenioDraftsRecordExtraFieldsBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_record_extra_fields"
    class_config = "record-class"
    template = "drafts-record-extra-fields"


from pathlib import Path
from typing import Union

from oarepo_model_builder.builder import ModelBuilder
from oarepo_model_builder.schema import ModelSchema
from oarepo_model_builder.profiles import Profile


class DraftsProfile(Profile):

    def build(
            self,
            model: ModelSchema,
            output_directory: Union[str, Path],
            builder: ModelBuilder,
    ):
        builder.build(model, output_directory)


import os

import lazy_object_proxy

from oarepo_model_builder.model_preprocessors import ModelPreprocessor
from oarepo_model_builder.utils.camelcase import snake_case


class InvenioDraftsDraftModelPreprocessor(ModelPreprocessor):
    TYPE = "invenio_drafts_draft"

    def transform(self, schema, settings):
        model = schema.current_model
        #alternatively record_prefix itself could be edited by adding "Draft" to it, it should work with editing the places where the unedited one is used for now (schema and mapping as far as i'm aware)
        model.setdefault("pid-field-args", ["create=True","delete=False"])
        model.setdefault("record-metadata-table-name", lazy_object_proxy.Proxy(lambda: f"{model.record_prefix.lower()}_metadata_draft"))
        model.setdefault("index-name", lazy_object_proxy.Proxy(lambda: f"{snake_case(model.record_prefix)}-draft-{os.path.basename(model.mapping_file).replace('.json', '')}"))
        model.setdefault("record-class", lazy_object_proxy.Proxy(lambda: f"{model.record_records_package}.api.{model.record_prefix}Draft"))
        model.setdefault("record-metadata-class", lazy_object_proxy.Proxy(lambda: f"{model.record_records_package}.models.{model.record_prefix}DraftMetadata"))
        model.setdefault("record-parent-class", "invenio_drafts_resources.records.api.Draft")
        model.setdefault("record-metadata-parent-class", "invenio_drafts_resources.records.DraftMetadataBase")


TEMPLATES = {
    "drafts-record-extra-fields": "templates/invenio_drafts_record_extra_fields.py.jinja2",                            #adds drafts extension fields to record
    "drafts-record-metadata-extra-fields": "templates/invenio_drafts_record_metadata_extra_fields.py.jinja2",  # adds parent specific fields to model metadata classes

    "drafts-parent-record": "templates/invenio_drafts_parent_record.py.jinja2",              #adds draft parent record class
    "drafts-parent-metadata": "templates/invenio_drafts_parent_metadata.py.jinja2",            #adds draft parent metadata classes

    "drafts-record-service-config": "templates/invenio_drafts_record_service_config.py.jinja2",
    "drafts-parent-state": "templates/invenio_drafts_parent_state.py.jinja2"
}


from oarepo_model_builder.builders import OutputBuilder
from oarepo_model_builder.outputs.cfg import CFGOutput


class InvenioDraftsSetupCfgBuilder(OutputBuilder):
    TYPE = "invenio_drafts_setup_cfg"

    def finish(self):
        super().finish()

        output: CFGOutput = self.builder.get_output("cfg", "setup.cfg")

        output.add_dependency(
            "invenio-drafts-resources", ">=1.0.4"
        )






from oarepo_model_builder.model_preprocessors import ModelPreprocessor

class InvenioDraftsBaseClassesModelPreprocessor(ModelPreprocessor):
    TYPE = "invenio_drafts_base_classes"

    def transform(self, schema, settings):
        self.set_default_and_append_if_not_present(
            schema.current_model,
            "record-metadata-bases",
            [],
            "invenio_drafts_resources.records.ParentRecordMixin"
        )




from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder


class InvenioDraftsParentRecordBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_parent_record"
    class_config = "drafts-parent-record-class"
    template = "drafts-parent-record"


from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder


class InvenioDraftsParentMetadataBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_parent_metadata"
    class_config = "drafts-parent-record-metadata-class"
    template = "drafts-parent-metadata"



import marshmallow as ma
from marshmallow import fields
from oarepo_model_builder.validation.model_validation import model_validator


class DraftAddonClasses(ma.Schema):
    drafts_parent_record_class = fields.String(data_key="drafts-parent-record-class", required=False)
    drafts_parent_record_parent_class = fields.String(data_key="drafts-parent-record-parent-class", required=False)
    drafts_parent_record_metadata_class = fields.String(data_key="drafts-parent-record-metadata-class", required=False)
    drafts_parent_state_class = fields.String(data_key="drafts-parent-state-class", required=False)
    drafts_parent_schema_name = fields.String(data_key="drafts-parent-schema-name", required=False)
    drafts_parent_schema_file = fields.String(data_key="drafts-parent-schema-file", required=False)


class DraftsModelSchema(ma.Schema):
    drafts = fields.Nested(lambda: model_validator.validator_class("model")())


validators = {"model": DraftAddonClasses, "root": DraftsModelSchema}



from oarepo_model_builder.model_preprocessors import ModelPreprocessor

class InvenioDraftsRecordModelPreprocessor(ModelPreprocessor):
    TYPE = "invenio_drafts_records"

    def transform(self, schema, settings):
        model = schema.current_model
        model.setdefault("plugins", {"builder": {"disable": ["invenio_tests_resource", "invenio_tests_service"]}})
        model.setdefault("drafts-parent-record-parent-class",
                         "invenio_drafts_resources.records.api.ParentRecord")
        model.setdefault("record-resource-parent-class", "invenio_drafts_resources.resources.RecordResource")
        model.setdefault("record-resource-config-parent-class", "invenio_drafts_resources.resources.RecordResourceConfig")
        model.setdefault("record-service-parent-class", "invenio_drafts_resources.services.RecordService")
        model.setdefault("record-service-config-parent-class", "invenio_drafts_resources.services.RecordServiceConfig")
        model.setdefault("record-parent-class", "invenio_drafts_resources.records.api.Record")
        model.setdefault("record-metadata-parent-class", "invenio_records.models.RecordMetadataBase")


import os

import lazy_object_proxy

from oarepo_model_builder.model_preprocessors import ModelPreprocessor


class InvenioDraftsCommonModelPreprocessor(ModelPreprocessor):
    # shared among the model and drafts profile run
    TYPE = "invenio_drafts_common"

    def transform(self, schema, settings):
        model = schema.current_model

        model.setdefault("pid-field-provider", "DraftRecordIdProviderV2")
        model.setdefault("pid-field-imports", [{"import": "invenio_records_resources.records.systemfields.pid.PIDField"},
                                               {"import": "invenio_records_resources.records.systemfields.pid.PIDFieldContext"},
                                               {"import": "invenio_drafts_resources.records.api.DraftRecordIdProviderV2"}])

        model.setdefault("drafts-parent-state-class", lazy_object_proxy.Proxy(lambda: f"{model.record_records_package}.models.ParentState"))
        model.setdefault("drafts-parent-record-metadata-class",
                         lazy_object_proxy.Proxy(lambda: f"{model.record_records_package}.models.{model.record_prefix}ParentRecordMetadata"))
        model.setdefault("drafts-parent-record-class",
                         lazy_object_proxy.Proxy(lambda: f"{model.record_records_package}.api.{model.record_prefix}ParentRecord"))
        model.setdefault("drafts-parent-schema-name", "parent-v1.0.0.json")
        model.setdefault("drafts-parent-schema-file", lazy_object_proxy.Proxy(lambda: os.path.join(
                model.package_path, "records", "jsonschemas", model.drafts_parent_schema_name
            )))
        model.setdefault("record-pid-provider-parent-class", "invenio_drafts_resources.records.api.DraftRecordIdProviderV2")


from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder
from oarepo_model_builder.outputs.python import PythonOutput
from oarepo_model_builder.utils.hyphen_munch import HyphenMunch


class InvenioDraftsTestResourcesBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_test_resources"
    template = "drafts-test-resources"
    MODULE = "tests.test_resources"

    def finish(self, **extra_kwargs):
        module = self.MODULE
        python_path = self.module_to_path(module)

        self.process_template(
            python_path,
            self.template,
            current_package_name=module,
            **extra_kwargs,
        )
    def process_template(self, python_path, template, **extra_kwargs):
        if self.parent_modules:
            self.create_parent_modules(python_path)
        output: PythonOutput = self.builder.get_output("python", python_path)
        context = HyphenMunch(
            settings=self.settings, current_model=self.current_model, model=self.schema.model, **extra_kwargs
        )
        template = self.call_components(
            "invenio_before_python_template", template, context=context
        )
        output.merge(template, context)


from oarepo_model_builder.invenio.invenio_base import InvenioBaseClassPythonBuilder


class InvenioDraftsRecordServiceConfigBuilder(InvenioBaseClassPythonBuilder):
    TYPE = "invenio_drafts_record_service_config"
    class_config = "record-service-config-class"
    template = "drafts-record-service-config"

